unit dscbmp;

interface

//------------------------------------------------------------------------------
// TDSColorizeBitmap Component Version 1.2
// By Jacob Copsey
// Copyright  1997 Jacob Copsey.
// All rights reserved.
//------------------------------------------------------------------------------
// Visit the Digital Solutions web page at:
//    http://www.cloudnet.com/~jcop/
//------------------------------------------------------------------------------
// Use of this component is free for use in freeware and shareware applications.
// Please contact us by e-mail (jcop@cloudnet.com) for permission to use this
// component in any commercial applications.
//------------------------------------------------------------------------------

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TDSColorizeBitmap = class(TGraphicControl)
  private
    fBitmap:     TBitmap;
    fDrawBitmap: TBitmap;
    fColor:      TColor;
    fForeColor:  TColor;
    fForeDither: Boolean;
    fSkipCreate: Boolean;
    fBmpMade:    Boolean;
    procedure CreateInternalBitmap;
    procedure SetBitmap(Value: TBitmap);
    procedure SetForeDither(Value: Boolean);
    procedure SetForeColor(Value: TColor);
    procedure SetColor(Value: TColor);
    procedure InvertBitmap(Bmp: TBitmap);
  protected
    procedure Paint; override;
  public
    procedure SetParams(Color, ForeColor: TColor; ForeDither: Boolean);
  published
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Align;
    property Color: TColor read fColor write SetColor;
    property ForeColor: TColor read fForeColor write SetForeColor;
    property ForeDither: Boolean read fForeDither write SetForeDither;
    property Bitmap: TBitmap read fBitmap write SetBitmap;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('DigiS', [TDSColorizeBitmap]);
end;


constructor TDSColorizeBitmap.Create(AOwner: TComponent);
begin
   fSkipCreate := true;
   width := 50;
   height := 50;
   fForeColor := clBtnHighlight;
   fColor := clBtnFace;
   fForeDither := True;
   fBitmap := TBitmap.Create;
   fBitmap.Monochrome := true;
   fDrawBitmap := TBitmap.Create;
   inherited;
end;


destructor TDSColorizeBitmap.Destroy;
begin
   fBitmap.Free;
   fDrawBitmap.Free;
   inherited;
end;


procedure TDSColorizeBitmap.Paint;
var
   x, y, xc, yc, w, h: integer;
begin
   w := fBitmap.Width;
   h := fBitmap.Height;
   if (w > 0) and (h > 0) then begin
      if not fBmpMade then begin
         fSkipCreate := false;
         CreateInternalBitmap;
      end;
      xc := (width div w);
      if ((width mod w) > 0) and (xc > 0) then inc(xc);
      yc := (height div h);
      if ((height mod h) > 0) and (yc > 0) then inc(yc);
      for x := 0 to xc do
         for y := 0 to yc do
            canvas.copyrect(rect(x * w, y * h, (x * w) + w, (y * h) + h),
                            fDrawBitmap.Canvas, Rect(0, 0, w, h));
   end
   else if csDesigning in ComponentState then begin
      canvas.pen.style := psDot;
      canvas.pen.color := clBlack;
      canvas.brush.style := bsSolid;
      canvas.brush.color := fColor;
      canvas.rectangle(0, 0, width, height);
   end
   else begin
      canvas.pen.style := psClear;
      canvas.brush.style := bsSolid;
      canvas.brush.color := fColor;
      canvas.rectangle(0, 0, width, height);
   end;
end;


procedure TDSColorizeBitmap.InvertBitmap(Bmp: TBitmap);
var
   TTemp: TBitmap;
begin
   Bmp.Canvas.CopyMode := cmDstInvert;
   Bmp.Canvas.CopyRect(rect(0, 0, Bmp.Width, Bmp.Height),
                       Bmp.Canvas, rect(0, 0, Bmp.Width, Bmp.Height));
end;


procedure TDSColorizeBitmap.SetForeDither(Value: Boolean);
begin
   if Value <> fForeDither then begin
      fForeDither := Value;
      CreateInternalBitmap;
      Invalidate;
   end;
end;


procedure TDSColorizeBitmap.SetForeColor(Value: TColor);
begin
   if Value <> fForeColor then begin
      fForeColor := Value;
      CreateInternalBitmap;
      Invalidate;
   end;
end;


procedure TDSColorizeBitmap.SetColor(Value: TColor);
begin
   if Value <> fColor then begin
      fColor := Value;
      CreateInternalBitmap;
      Invalidate;
   end;
end;


procedure TDSColorizeBitmap.SetBitmap(Value: TBitmap);
begin
   if not (Value = fBitmap) then begin
      fBitmap.Assign(Value);
      CreateInternalBitmap;
      Invalidate;
   end;
end;


procedure TDSColorizeBitmap.CreateInternalBitmap;
var
   lTempBitmap:   TBitmap;
   lTempBitmap2:  TBitmap;
   lDitherBitmap: TBitmap;
   lCRect:        TRect;
   x, y, w, h:    Integer;
begin
   if (not fSkipCreate) then begin
      w := fBitmap.Width;
      h := fBitmap.Height;
      lTempBitmap := TBitmap.Create;
      lTempBitmap.Width := w;
      lTempBitmap.Height := h;
      lTempBitmap2 := TBitmap.Create;
      lTempBitmap2.Width := w;
      lTempBitmap2.Height := h;
      lCRect := rect(0, 0, w, h);
      with lTempBitmap.Canvas do begin
         Brush.Style := bsSolid;
         Brush.Color := Color;
         FillRect(lCRect);
         CopyMode := cmSrcInvert;
         CopyRect(lCRect, fBitmap.Canvas, lCRect);
         InvertBitmap(fBitmap);
         CopyMode := cmSrcPaint;
         CopyRect(lCRect, fBitmap.Canvas, lCRect);
         InvertBitmap(lTempBitmap);
         CopyMode := cmSrcInvert;
         CopyRect(lCRect, fBitmap.Canvas, lCRect);
         InvertBitmap(fBitmap);
      end;
      with lTempBitmap2.Canvas do begin
         Brush.Style := bsSolid;
         Brush.Color := clBlack;
         FillRect(lCRect);
         if fForeDither then begin
            InvertBitmap(fBitmap);
            lDitherBitmap := TBitmap.Create;
            lDitherBitmap.Width := 8;
            lDitherBitmap.Height := 8;
            with lDitherBitmap.canvas do begin
               for x := 0 to 7 do
                  for y := 0 to 7 do
                     if ((x mod 2 = 0) and (y mod 2 > 0)) or
                        ((x mod 2 > 0) and (y mod 2 = 0)) then
                           pixels[x,y] := fForeColor
                     else
                           pixels[x,y] := Color;
            end;
            Brush.Bitmap := lDitherBitmap;
         end
         else begin
            Brush.Style := bsSolid;
            Brush.Color := fForeColor;
         end;
         if not fForeDither then InvertBitmap(fBitmap);
         CopyMode := cmPatPaint;
         CopyRect(lCRect, fBitmap.Canvas, lCRect);
         if fForeDither then lDitherBitmap.Free;
         CopyMode := cmSrcInvert;
         CopyRect(lCRect, fBitmap.Canvas, lCRect);
      end;
      lTempBitmap.Canvas.CopyMode := cmSrcInvert;
      lTempBitmap.Canvas.Copyrect(lCRect, lTempBitmap2.Canvas, lCRect);
      InvertBitmap(fBitmap);
      lTempBitmap.Canvas.CopyMode := cmSrcErase;
      lTempBitmap.Canvas.Copyrect(lCRect, fBitmap.Canvas, lCRect);
      InvertBitmap(fBitmap);
      lTempBitmap.Canvas.CopyMode := cmSrcInvert;
      lTempBitmap.Canvas.Copyrect(lCRect, lTempBitmap2.Canvas, lCRect);
      InvertBitmap(lTempBitmap);
      InvertBitmap(fBitmap);
      fDrawBitmap.Assign(lTempBitmap);
      lTempBitmap.Free;
      fBmpMade := True;
      invalidate;
   end;
end;


procedure TDSColorizeBitmap.SetParams(Color, ForeColor: TColor; ForeDither: Boolean);
begin
   fColor := Color;
   fForeColor := ForeColor;
   fForeDither := ForeDither;
   CreateInternalBitmap;
   Invalidate;
end;


end.
